==> kdtree-0.0.1.tgz: package/src/node-kdtree.cc <==
/**
 * This file is part of node-kdtree, a node.js Addon for working with kd-trees.
 * Copyright (C) 2011 Justin Ethier <justinethier@github>
 *
 * Please use github to submit patches and bug reports:
 * https://github.com/justinethier/node-kdtree
 */

#include <v8.h>
#include <node.h>
#include <node_events.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <kdtree.h>

using namespace v8;
using namespace node;

/**
 * Free memory allocated to the 'data' portion of a node...
 */
void freeNodeData(void *data){
  if (data != NULL) {
    // Release this persistent handle's storage cell
    Persistent<Value> hData = Persistent<Value>::Persistent((Value *)data);
    hData.Dispose();
  }
}

/**
 * The KDTree add-on
 */
class KDTree : public ObjectWrap {
  public:
    static void
    Initialize (v8::Handle<v8::Object> target){
        HandleScope scope;

        Local<FunctionTemplate> t = FunctionTemplate::New(New);

        t->InstanceTemplate()->SetInternalFieldCount(1);
        t->SetClassName(String::NewSymbol("KDTree"));

        NODE_SET_PROTOTYPE_METHOD(t, "dimensions", Dimensions);
        NODE_SET_PROTOTYPE_METHOD(t, "insert", Insert);
        NODE_SET_PROTOTYPE_METHOD(t, "nearest", Nearest);
//
//      FUTURE: 2 methods below to get the nearest point, and nearest data.
//              Ideally could be written in js, but C++ is fine too. Idea is that most usage
//              would probably just be for one or the other, and not the combined list...
//
//        NODE_SET_PROTOTYPE_METHOD(t, "nearestPoint", NearestPoint);
//        NODE_SET_PROTOTYPE_METHOD(t, "nearestValue", NearestValue);
//
        NODE_SET_PROTOTYPE_METHOD(t, "nearestRange", NearestRange);

        target->Set(String::NewSymbol("KDTree"), t->GetFunction());
    }

    /**
     * Getter for tree's dimensions.
     *
     * @return Dimensions of the tree's points.
     */
    int Dimensions(){
      return dim_;
    }

    /**
     * Insert a set of points into the tree.
     *
     * An optional data argument is present as well; 
     * if len == dim_, then the data param will be ignored.
     *
     * @param pos   An array of points
     * @param len   Number of points in the array
     * @param data  Optional data argument
     *
     * @return true if the point was inserted successfully, false otherwise
     */ 
    bool Insert(const double *pos, int len, Handle<Value> data){
      if (len != dim_ && len != dim_ + 1){
        ThrowException(Exception::Error(String::New("Insert(): Wrong number of parameters."))); 
        // FUTURE: Passed: " + len + " Expected: " + dim_)));
      }

      if (len == dim_)
        return (kd_insert(kd_, pos, NULL) == 0);
      return (kd_insert(kd_, pos, *data) == 0);
    }

    /**
     * Find the point nearest to the given point.
     *
     * @param pos   An array of points
     * @param len   Number of points in the array
     *
     * @return An array containing the nearest point, or an empty array if no point is found.
     *         If a data element was provided for the nearest point, it will be the last
     *         member of the returned array.
     */ 
    Handle<Value>
    Nearest(const double *pos, int len){
      int rpos;
      void *pdata;
      kdres *results = kd_nearest(kd_, pos);
      Local<Array> rv = Array::New(dim_ + 1);

      if (len != dim_){
        ThrowException(Exception::Error(String::New("Nearest(): Wrong number of parameters."))); 
        // FUTURE: Passed: " + len + " Expected: " + dim_)));
      }

      if (results != NULL) {
        double *respos = (double *)(malloc(sizeof(double) * dim_));
        pdata = (void *)kd_res_item(results, respos); 

        for(rpos = 0; rpos < dim_; rpos++){
          rv->Set(rpos, Number::New(respos[rpos])); 
        }

        // Append data element, if present
        if (pdata != NULL) {
          Persistent<Value> hdata = Persistent<Value>::Persistent((Value *)pdata);
          rv->Set(dim_, hdata); 
        }

        free(respos);
        kd_res_free(results);
      }
      return rv;
    }

    /**
     * Find the points nearest to the given point, within a given range.
     *
     * @param pos   An array of points
     * @param len   Number of points in the array
     * @param range Range in which to search for points
     *
     * @return An array containing the nearest points, or an empty array if no point is found.
     *         If a data element was provided for any point, it will be the last
     *         member of that point's returned array.
     */ 
    Handle<Value> 
    NearestRange(const double *pos, int len, double range){
      int rpos, i = 0;
      void *pdata;
      kdres *results = kd_nearest_range(kd_, pos, range);
      Local<Array> rv = Array::New();

      if (len != dim_){
        ThrowException(Exception::Error(String::New("Nearest(): Wrong number of parameters."))); 
        // FUTURE: Passed: " + len + " Expected: " + dim_)));
      }

      while (!kd_res_end( results )){
        Local<Array> rvItem = Array::New(dim_ + 1);
        double *respos = (double *)(malloc(sizeof(double) * dim_));
        pdata = (void *)kd_res_item(results, respos); 

        for(rpos = 0; rpos < dim_; rpos++){
          rvItem->Set(rpos, Number::New(respos[rpos])); 
        }

        // Append data element, if present
        if (pdata != NULL) {
          Persistent<Value> hdata = Persistent<Value>::Persistent((Value *)pdata);
          rvItem->Set(dim_, hdata); 
        }

        rv->Set(i++, rvItem);
        free(respos);

        // Move to next result entry
        kd_res_next( results );
      }

      kd_res_free(results);
      return rv;
    }

  protected:

    static Handle<Value>
    Dimensions (const Arguments& args){
        KDTree *kd = ObjectWrap::Unwrap<KDTree>(args.This());
        HandleScope scope;

        return Number::New(kd->Dimensions());
    }

    /**
     * Wrapper for Insert()
     */
    static Handle<Value>
    Insert(const Arguments& args){
        KDTree *kd = ObjectWrap::Unwrap<KDTree>(args.This());
        HandleScope scope;

// TODO: range checking

      double *pos = (double *)(malloc(sizeof(double) * args.Length()));
      for (int i = 0; i < args.Length(); i++){
        pos[i] = args[i]->NumberValue();
      }

      Handle<Value> result = Boolean::New( kd->Insert(pos, args.Length(),
          Persistent<Value>::New(args[ args.Length() - 1])));
      free(pos);
      return result;
    }

    /**
     * Wrapper for Nearest()
     */ 
    static Handle<Value>
    Nearest(const Arguments& args){
      KDTree *kd = ObjectWrap::Unwrap<KDTree>(args.This());
      HandleScope scope;

// TODO: range checking

      double *pos = (double *)(malloc(sizeof(double) * args.Length()));
      for (int i = 0; i < args.Length(); i++){
        pos[i] = args[i]->NumberValue();
      }

      Handle<Value> result = kd->Nearest(pos, args.Length()); 
        
      free(pos);
      return result;
    }

/* FUTURE:
    static Handle<Value>
    NearestPoint(const Arguments& args){
      return KDTree::Nearest(args);
    }

    static Handle<Value>
    NearestValue(const Arguments& args){
      HandleScope scope;
//      Handle<Array> array = Handle<Array>::Cast(KDTree::Nearest(args));
      Local<Array> arry = Array::New(KDTree::Nearest(args));
      Local<Array> result = Array::New(); 
      if (arry->Length() > 0){
        result->Set(0, String::New("todo")); //arry[arry->Length() - 1]->Value());
      }

      return result;
    }
*/
    static Handle<Value>
    NearestRange(const Arguments& args){
      KDTree *kd = ObjectWrap::Unwrap<KDTree>(args.This());
      HandleScope scope;
      
// TODO: range checking here

      double *pos = (double *)(malloc(sizeof(double) * args.Length() - 1));
      for (int i = 0; i < args.Length() - 1; i++){
        pos[i] = args[i]->NumberValue();
      }

      Handle<Value> result = kd->NearestRange(pos, args.Length() - 1, args[args.Length() - 1]->NumberValue()); 
        
      free(pos);
      return result;
    }

    /**
     * "External" constructor called by the Addon framework
     */
    static Handle<Value>
    New (const Arguments& args){
        HandleScope scope;

        int dimension = 3; // Default
        if (args.Length() > 0){
          dimension = args[0]->Int32Value();
        }

        KDTree *kd = new KDTree(dimension);
        kd->Wrap(args.This());

        return args.This();
    }

    /**
     * Constructor
     *
     * @param dim   Dimensions of each point in the tree
     */
    KDTree (int dim) : ObjectWrap (){
        kd_ = kd_create(dim);
        dim_ = dim;
        kd_data_destructor(kd_, freeNodeData);
    }

    /**
     * Destructor
     */
    ~KDTree(){
        if (kd_ != NULL){
            kd_free(kd_);
        }
    }

  private:
    /**
     * Pointer to the tree itself
     */
    kdtree* kd_;

    /**
     * Dimension of each point in the tree
     */
    int dim_;
};

/**
 * Entry point required by node.js framework
 */
extern "C" void
init (Handle<Object> target)
{
    HandleScope scope;
    KDTree::Initialize(target);
}

==> kdtree-0.0.2.tgz: package/src/node-kdtree.cc <==
/**
 * This file is part of node-kdtree, a node.js Addon for working with kd-trees.
 * Copyright (C) 2011 Justin Ethier <justinethier@github>
 *
 * Please use github to submit patches and bug reports:
 * https://github.com/justinethier/node-kdtree
 */

#include <v8.h>
#include <node.h>
#include <node_events.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <kdtree.h>

using namespace v8;
using namespace node;

/**
 * Free memory allocated to the 'data' portion of a node...
 */
void freeNodeData(void *data){
  if (data != NULL) {
    // Release this persistent handle's storage cell
    Persistent<Value> hData = Persistent<Value>::Persistent((Value *)data);
    hData.Dispose();
  }
}

/**
 * The KDTree add-on
 */
class KDTree : public ObjectWrap {
  public:
    static void
    Initialize (v8::Handle<v8::Object> target){
        HandleScope scope;

        Local<FunctionTemplate> t = FunctionTemplate::New(New);

        t->InstanceTemplate()->SetInternalFieldCount(1);
        t->SetClassName(String::NewSymbol("KDTree"));

        NODE_SET_PROTOTYPE_METHOD(t, "dimensions", Dimensions);
        NODE_SET_PROTOTYPE_METHOD(t, "insert", Insert);
        NODE_SET_PROTOTYPE_METHOD(t, "nearest", Nearest);
        NODE_SET_PROTOTYPE_METHOD(t, "nearestPoint", NearestPoint);
        NODE_SET_PROTOTYPE_METHOD(t, "nearestValue", NearestValue);
        NODE_SET_PROTOTYPE_METHOD(t, "nearestRange", NearestRange);

        target->Set(String::NewSymbol("KDTree"), t->GetFunction());
    }

    /**
     * Getter for tree's dimensions.
     *
     * @return Dimensions of the tree's points.
     */
    int Dimensions(){
      return dim_;
    }

    /**
     * Insert a set of points into the tree.
     *
     * An optional data argument is present as well; 
     * if len == dim_, then the data param will be ignored.
     *
     * @param pos   An array of points
     * @param len   Number of points in the array
     * @param data  Optional data argument
     *
     * @return true if the point was inserted successfully, false otherwise
     */ 
    bool Insert(const double *pos, int len, Handle<Value> data){
      if (len != dim_ && len != dim_ + 1){
        ThrowException(Exception::Error(String::New("Insert(): Wrong number of parameters."))); 
        // FUTURE: Passed: " + len + " Expected: " + dim_)));
      }

      if (len == dim_)
        return (kd_insert(kd_, pos, NULL) == 0);
      return (kd_insert(kd_, pos, *data) == 0);
    }

    /**
     * Find the point nearest to the given point.
     *
     * @param pos   An array of points
     * @param len   Number of points in the array
     *
     * @return An array containing the nearest point, or an empty array if no point is found.
     *         If a data element was provided for the nearest point, it will be the last
     *         member of the returned array.
     */ 
    Handle<Value>
    Nearest(const double *pos, int len){
      HandleScope scope;
      int rpos;
      void *pdata;
      kdres *results = kd_nearest(kd_, pos);
      Local<Array> rv = Array::New(dim_ + 1);

      if (len != dim_){
        ThrowException(Exception::Error(String::New("Nearest(): Wrong number of parameters."))); 
        // FUTURE: Passed: " + len + " Expected: " + dim_)));
      }

      if (results != NULL) {
        double *respos = (double *)(malloc(sizeof(double) * dim_));
        pdata = (void *)kd_res_item(results, respos); 

        for(rpos = 0; rpos < dim_; rpos++){
          rv->Set(rpos, Number::New(respos[rpos])); 
        }

        // Append data element, if present
        if (pdata != NULL) {
          Persistent<Value> hdata = Persistent<Value>::Persistent((Value *)pdata);
          rv->Set(dim_, hdata); 
        }

        free(respos);
        kd_res_free(results);
      }
//      return rv;
      return scope.Close(rv);
//      TODO: perhaps need to call this, to support NearestValue:
//      scope.Close(Number::New(getpid()));
//
//      perhaps everywhere there is a value returned from handlescope, should close scope and return it??
    }

    /**
     * Find the points nearest to the given point, within a given range.
     *
     * @param pos   An array of points
     * @param len   Number of points in the array
     * @param range Range in which to search for points
     *
     * @return An array containing the nearest points, or an empty array if no point is found.
     *         If a data element was provided for any point, it will be the last
     *         member of that point's returned array.
     */ 
    Handle<Value> 
    NearestRange(const double *pos, int len, double range){
      int rpos, i = 0;
      void *pdata;
      kdres *results = kd_nearest_range(kd_, pos, range);
      Local<Array> rv = Array::New();

      if (len != dim_){
        ThrowException(Exception::Error(String::New("Nearest(): Wrong number of parameters."))); 
        // FUTURE: Passed: " + len + " Expected: " + dim_)));
      }

      while (!kd_res_end( results )){
        Local<Array> rvItem = Array::New(dim_ + 1);
        double *respos = (double *)(malloc(sizeof(double) * dim_));
        pdata = (void *)kd_res_item(results, respos); 

        for(rpos = 0; rpos < dim_; rpos++){
          rvItem->Set(rpos, Number::New(respos[rpos])); 
        }

        // Append data element, if present
        if (pdata != NULL) {
          Persistent<Value> hdata = Persistent<Value>::Persistent((Value *)pdata);
          rvItem->Set(dim_, hdata); 
        }

        rv->Set(i++, rvItem);
        free(respos);

        // Move to next result entry
        kd_res_next( results );
      }

      kd_res_free(results);
      return rv;
    }

  protected:

    static Handle<Value>
    Dimensions (const Arguments& args){
        KDTree *kd = ObjectWrap::Unwrap<KDTree>(args.This());
        HandleScope scope;

        return Number::New(kd->Dimensions());
    }

    /**
     * Wrapper for Insert()
     */
    static Handle<Value>
    Insert(const Arguments& args){
        KDTree *kd = ObjectWrap::Unwrap<KDTree>(args.This());
        HandleScope scope;

// TODO: range checking

      double *pos = (double *)(malloc(sizeof(double) * args.Length()));
      for (int i = 0; i < args.Length(); i++){
        pos[i] = args[i]->NumberValue();
      }

      Handle<Value> result = Boolean::New( kd->Insert(pos, args.Length(),
          Persistent<Value>::New(args[ args.Length() - 1])));
      free(pos);
      return result;
    }

    /**
     * Wrapper for Nearest()
     */ 
    static Handle<Value>
    Nearest(const Arguments& args){
      KDTree *kd = ObjectWrap::Unwrap<KDTree>(args.This());
      HandleScope scope;

// TODO: range checking

      double *pos = (double *)(malloc(sizeof(double) * args.Length()));
      for (int i = 0; i < args.Length(); i++){
        pos[i] = args[i]->NumberValue();
      }

      Handle<Value> result = kd->Nearest(pos, args.Length()); 
        
      free(pos);
      return scope.Close(result);
    }

    /**
     * A shortcut method for Nearest, that returns an array containing only point data.
     * If a value is present, it will NOT be returned in the result array.
     */
    static Handle<Value>
    NearestPoint(const Arguments& args){
      HandleScope scope;
      Handle<Array> nearest = ((KDTree::Nearest(args))).As<Array>();
      int dim = KDTree::Dimensions(args).As<Number>()->Value();

      Local<Array> result = Array::New(dim); 
      if (nearest->Length() > 0 &&    // Data present
          nearest->Length() >= dim) { // Points present
         for (int i = 0; i < dim; i++) {
           result->Set(i, nearest->Get(i));
         }
      }

      return scope.Close(result);
    }

    // TODO: Return as a scalar instead of an array
    /**
     * A shortcut method for Nearest, that returns an array containing only the point's value.
     * If a value is not present or no point was found, the returned array will be empty.
     */
    static Handle<Value>
    NearestValue(const Arguments& args){
      HandleScope scope;
      Handle<Array> nearest = ((KDTree::Nearest(args))).As<Array>();
      int dim = KDTree::Dimensions(args).As<Number>()->Value();

      Local<Array> result = Array::New(1); 
      if (nearest->Length() > 0 &&       // Data present
          nearest->Length() == (dim + 1)){ // Value present
        result->Set(0, nearest->Get(nearest->Length() - 1));
      }

      return scope.Close(result);
    }

    static Handle<Value>
    NearestRange(const Arguments& args){
      KDTree *kd = ObjectWrap::Unwrap<KDTree>(args.This());
      HandleScope scope;
      
// TODO: range checking here

      double *pos = (double *)(malloc(sizeof(double) * args.Length() - 1));
      for (int i = 0; i < args.Length() - 1; i++){
        pos[i] = args[i]->NumberValue();
      }

      Handle<Value> result = kd->NearestRange(pos, args.Length() - 1, args[args.Length() - 1]->NumberValue()); 
        
      free(pos);
      return result;
    }

    /**
     * "External" constructor called by the Addon framework
     */
    static Handle<Value>
    New (const Arguments& args){
        HandleScope scope;

        int dimension = 3; // Default
        if (args.Length() > 0){
          dimension = args[0]->Int32Value();
        }

        KDTree *kd = new KDTree(dimension);
        kd->Wrap(args.This());

        return args.This();
    }

    /**
     * Constructor
     *
     * @param dim   Dimensions of each point in the tree
     */
    KDTree (int dim) : ObjectWrap (){
        kd_ = kd_create(dim);
        dim_ = dim;
        kd_data_destructor(kd_, freeNodeData);
    }

    /**
     * Destructor
     */
    ~KDTree(){
        if (kd_ != NULL){
            kd_free(kd_);
        }
    }

  private:
    /**
     * Pointer to the tree itself
     */
    kdtree* kd_;

    /**
     * Dimension of each point in the tree
     */
    int dim_;
};

/**
 * Entry point required by node.js framework
 */
extern "C" void
init (Handle<Object> target)
{
    HandleScope scope;
    KDTree::Initialize(target);
}
